## Usage: mjplot ([<arguments>])
##
## If hold is on, add plot(s) to the current plot window made with
## mjplot. Otherwise clear the plot window before plotting.
##
## No arguments
## ============
## mjplot ()
##
## Replot previous plot made with mjplot (regardless of the hold state).
##
## Single numeric arguments
## ========================
## mjplot (y1)
## mjplot (y1, <format arguments>)
## mjplot (y1, <format arguments>, y2)
## mjplot (y1, <format arguments>, y2, <format arguments>, ...)
##
## If only one numeric argument is given or if single numeric arguments
## are separated by format arguments there are two possibilities:
##
## 1) Real numeric arguments
## -------------------------
## The numeric arguments are taken as sets of y coordinates and the x
## coordinates are taken to be the indices of the elements, starting
## with 1. For matrices, each column is taken as a set of y coordinates.
##
## 2) Complex numeric arguments
## ----------------------------
## The real parts is taken to be the x coordinates and the imaginary
## parts is taken to be the y coordinates.
##
## Pairs of numeric arguments
## ==========================
## mjplot (x1, y1)
## mjplot (x1, y1, <format arguments>)
## mjplot (x1, y1, x2, y2)
## mjplot (x1, y1, <format arguments>, x2, y2, ...)
##
## If the numeric arguments appear in pairs the imaginary parts are
## ignored.
##
## 1) The first argument is a vector and the second is a matrix
## ------------------------------------------------------------
## The vector is plotted versus the columns (or rows) of the matrix.
## (using whichever combination matches, with columns tried first.)
##
## 2) The first argument is a matrix and the second is a vector
## ------------------------------------------------------------
## The columns (or rows) of the matrix are plotted versus the vector.
## (using whichever combination matches, with columns tried first.)
##
## 3) Both arguments are vectors
## -----------------------------
## The elements of y are plotted versus the elements of x.
##
## 4) Both arguments are matrices
## ------------------------------
## The columns of y are plotted versus the columns of x. In this case,
## both matrices must have the same number of rows and columns and no
## attempt is made to transpose the arguments to make the number of rows
## match.
##
## 5) Both arguments are scalars
## -----------------------------
## A single point is plotted.
##
## Format arguments
## ================
## Some of the plot styles requires more than two columns of data (eg.
## errorbars). These styles are not supported by mjplot (yet).
##
## The format arguments can be given as line specifications and / or as
## switch / argument pairs.
##
## Line specifications
## -------------------
## The line specifications can be a combination of the following:
## "-"           lines style (default)
## "." or ":"    dots style
## "@"           points style
## "-@" or "-."  linespoints style
## "i"           impulses style
## "L"           steps style
## "#"           boxes style
## "~"           errorbars style
## "#~"          boxerrorbars style
## "n"           with n in 0...9, line type number
## "nm"          with n and m in 0...9, line type and point type number
## ";title;"     where "title" is the label for the key
##
## Formats useful for terminal x11:
## "o" or "d"    point type 1 (a dimond in x11)
## "+"           point type 2 (a plus sign in x11)
## "s"           point type 3 (a square in x11)
## "x"           point type 4 (a cross in x11)
## "^"           point type 5 (an upward pointing triangle in x11)
## "*", "<"  ">", "h", "p" or "v" 
##               point type 6 (a star in x11)
## "r"           line type 1 (red in x11)
## "g"           line type 2 (green in x11)
## "b"           line type 3 (blue in x11)
## "m"           line type 4 (magenta in x11)
## "c"           line type 5 (cyan in x11)
## "k", "w" or "y" 
##               line type 6 (brown in x11)
##
## Switch / argument
## -----------------
## The switch is always a string, the argument can always be a string
## and for some switches even numeric.
##
## Switch:       Style, LineStyle or LS
## - - - - - - - - - - - - - - - - - - -
## Description:  Set the style
## Argument:     String; see table below (abbreviations are allowed)
##
## Argument                                       Style
## ..........................................................
## "-", "--", "l", "lines", empty string,
##  empty matrix or spaces                        lines
## "@" or "points"                                points
## "-@", "@-", "-.", ".-", "lp" or "linespoints"  linespoints
## "i", "^" or "impulses"                         impulses
## ".", ":" or "dots"                             dots
## "L" or "steps"                                 steps
## "fsteps"                                       fsteps
## "histeps"                                      histeps
## "~" or "errorbars"                             errorbars
## "xerrorbars"                                   xerrorbars
## "yerrorbars"                                   yerrorbars
## "xyerrorbars"                                  xyerrorbars
## "#" or "boxes"                                 boxes
## "#~", "~#" or "boxerrorbars"                   boxerrorbars
## "boxxyerrorbars"                               boxxyerrorbars
## "financebars"                                  financebars
## "candlesticks"                                 candlesticks
## "vector"                                       vector
##
## Switch:      LineWidth or LW
## - - - - - - - - - - - - - - -
## Description: Set the line width
## Argument:    String or numeric
##
## The argument can be numeric or a string that can be converted to a
## numeric argument with the "str2num" function. The argument is
## truncated to form an integer. If the numeric argument is a row vector
## (or the string is converted to a row vector), the first element
## (column number 1) is used.
##
## Switch:      PointType, PT or Marker
## - - - - - - - - - - - - - - - - - - -
## Description: Set the point type
## Argument:    see LineWidth
##
## There are also some arguments useful for terminal x11 (abbreviations
## are allowed):
## "o" or "diamond"    point type 1 (a dimond in x11)
## "+"                 point type 2 (a plus sign in x11)
## "square"            point type 3 (a square in x11)
## "x"                 point type 4 (a cross in x11)
## "^"                 point type 5 (an upward pointing triangle in x11)
## "*", ".", "<", ">", "hexagram", "pentagram" or "v" 
##                     point type 6 (a star in x11)
##
## Switch:      PointSize, PS or MarkerSize
## - - - - - - - - - - - - - - - - - - - - -
## Description: Set the point size
## Argument:    see LineWidth
##
## Switch:      Title
## - - - - - - - - - -
## Description: Set the title (also known as key or legend) for a plot. 
## Argument:    String.
##
## Switch:      LineType, LT, Color, MarkerEdgeColor or MarkerFaceColor
## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
## Description: Set the line type.
## Argument:    String or numeric.
##
## If the numeric argument is a scalar or the string argument is
## converted to a scalar with the "str2num" function, the value is
## truncated to form an integer, and the line type is set to that
## integer.
##
## If the numeric argument is a row vector with three colomns or the
## string argument is converted to a row vector with three columns with
## the "str2num" function, each element is rounded to the nearest
## integer. The line type is set as in the table below.
##
## Rounded argument        Line type
## ......................................................
## [0, 0, 0]               0 (black dot line in term x11)
## [1, 0, 0]               1 (red in term x11)
## [0, 1, 0]               2 (green in term x11)
## [0, 0, 1]               3 (blue in term x11)
## [1, 0, 1]               4 (magenta in term x11)
## [0, 1, 1]               5 (cyan in term x11)
## [1, 1, 0] or [1, 1, 1]  6 (brown in term x11)  
##
## There are also some string arguments useful in terminal x11
## (abbreviations are allowed):
##
## Argument                Line type
## ..............................................
## "red"                   1 (red in term x11)    
## "green"                 2 (green in term x11)  
## "b" or "blue"           3 (blue in term x11)   
## "magenta"               4 (magenta in term x11)
## "cyan"                  5 (cyan in term x11)   
## "white", "k", "black", "yellow" or "brown"
##                         6 (brown in term x11)
## empty matrix, "auto" or "none"
##                         do nothing
##
## Switch:      Smooth
## - - - - - - - - - -
## Description: Interpolation or approximation of data 
## Argument:    String: "unique", "csplines" (default), "acsplines",
##                      "bezier" or "sbezier" (or abbreviation).
##                      See "help smooth" in gnuplot for details.
##
## Switch:      Axes
## - - - - - - - - -
## Description: Chose a set of axes 
## Argument:    String: "x1y1", "x1y2", "x2y1", or x2y2"
##
## There are four possible sets of axes available:
## "x1y1" refers to the axes on the bottom and left; "x2y2" to those on
## the top and right; "x1y2" to those on the bottom and right; and
## "x2y1" to those on the top and left.
##
## See "help plot" in gnuplot for details.
##
## Switch:      Using
## - - - - - - - - - -
## Description: Data modifier
## Argument:    String or numeric
## 
## A numeric argument is converted to a string with a colon to separate
## the elements (eg [1, 2] is converted to "1:2").
##
## A string argument is not modified (except for a deblank).
##
## See "help using" in gnuplot.
##
## Note:
## For the moment mjplot only supports plot-styles that uses two columns
## of data. Only two columns are sent to gnuplot. For that reason the
## argument to "Using" can only contain the columns 1 and 2. 
##
## Eg:
## "Using", [2, 1]     plots x versus y instead of y versus x.
## "Using", "1:(2*$2)" plots 2*y versus x.
##
## Examples
## ========
## mjplot (x, y, "@12", x, y2, x, y3, "4", x, y4, "+")
##
##   y will be plotted with points of type 2 ("+") and color 1 (red).
##   y2 will be plotted with lines.
##   y3 will be plotted with lines of color 4.
##   y4 will be plotted with points which are "+"s.
##
## mjplot (b, "*")
##
##   b will be plotted with points of type "*".
##
## x = [1:10]';
## y = [x, 2*x, 3*x];
## mjplot (x, y, \
##         ["-"; "-@"; "@"], \
##         ["LineWidth"; "LineWidth"; "PointSize"], [2:4]', \
##         "Title", ["First column"; "Second"; "Third"]);
## 
##   The columns of y will be plotted with lines, linespoints and points
##   respectively. The line width of the first and second plot will be
##   two and three times the default line width respectively. The point
##   size of the third plot will be four times the default point size.
##   The titles will be "First column", "Second" and "Third".
##
## Note
## ====
## 'mjplot' uses an own gnuplot session (not the same as the builtin
## function 'gplot' uses). Commands like 'gset grid' will not work, use
## 'mjgraw ("set grid\n")' instead.
##
## 'mjplot' uses inline data (no temporary files). 'replot' will not
## work (not 'mjgraw ("replot\n")' either), use 'mjplot ()' instead. 
##
## 'mjplot' doesn't recognize error messages from gnuplot. This can
## cause problems if the hold-state is on or when 'mjplot' is called
## with no arguments.
##
## See also: mjgraw, mjcloseplot

## Mats Jansson, May 9 2000 - June 4 2000
## Report bugs to: mats@nera.no

function mjplot (...)
  dfi = do_fortran_indexing;
  eleo = empty_list_elements_ok;
  unwind_protect
    do_fortran_indexing = 1;
    empty_list_elements_ok = 0;
    global __mjplt_mjgraw_cmd__ = "plot";
    global __mjplt_il_data__ = "";
    caller = "mjplot";
        
    lm = length (__mjplt_mjgraw_cmd__);
    if (ishold),
      if (nargin && __mjplt_mjgraw_cmd__(lm) == "\n"),
        __mjplt_mjgraw_cmd__(lm) = ",";
      endif
    elseif (nargin),
      __mjplt_mjgraw_cmd__ = "plot";
      lm = length (__mjplt_mjgraw_cmd__);
      __mjplt_il_data__ = "";
    endif
    li = length (__mjplt_il_data__);
    
    if (nargin),
      try
        USING  =    "u";
        SMOOTH =    "s";
        AXES   =    "ax";
        TITLE  =    "t";
        WITH   =    "w";
        STYLE  =   ["l             "; \  ## LINES                      
                    "p             "; \  ## POINTS                  
                    "linesp        "; \  ## LINESPOINTS        
                    "i             "; \  ## IMPULSES                
                    "d             "; \  ## DOTS                    
                    "st            "; \  ## STEPS                   
                    "fsteps        "; \  ## FSTEPS             
                    "histeps       "; \  ## HISTEPS           
                    "e             "; \  ## ERRORBARS               
                    "xerrorbars    "; \  ## XERRORBARS     
                    "yerrorbars    "; \  ## YERRORBARS     
                    "xyerrorbars   "; \  ## XYERRORBARS   
                    "boxes         "; \  ## BOXES               
                    "boxer         "; \  ## BOXERRORBARS        
                    "boxxyerrorbars"; \  ## BOXXYERRORBARS
                    "financebars   "; \  ## FINANCEBARS   
                    "candlesticks  "; \  ## CANDLESTICKS  
                    "vector        "];   ## VECTOR            
        LINETYPE  = "lt";
        LINEWIDTH = "lw";
        POINTTYPE = "pt";
        POINTSIZE = "ps";
        
        default_fmt = sprintf ("%s %s", WITH, deblank (STYLE(1,:)));
        
        x_set = 0;
        y_set = 0;
        s_set = 0;
        next_x_set = 0;
        va_ind = 1;
        while (1),
          if (! next_x_set && va_ind <= nargin),
            x = va_arg ();
            ++va_ind;
            if (isstr (x)),
              error ("mjplot: expecting numeric argument");
            endif
            x_set = 1;
          elseif (next_x_set),
            x = next_x;
            x_set = 1;
            next_x_set = 0;
          else
            break;
          endif
          ## Now, we have one numeric argument (x).
          
          if (va_ind <= nargin),
            y = va_arg ();
            ++va_ind;
            if (isstr (y)),
              ## There was only one numeric argument (x), later x are
              ## going to be plotted as y-values with there indices as
              ## x-values or with the real part as x-values and the
              ## imaginary part as y-values.
              s = y;
              ## The first format specification. This can be a
              ## line-specification or a switch.
              s_set = 1;
            else
              ## There was two numeric arguments.
              y_set = 1;
            endif
          endif
          
          if (va_ind <= nargin && y_set),
            ## We have two numeric argument, maybe the next argument is
            ## a formatspecification.
            s = va_arg ();
            ++va_ind;
            if (! isstr (s)),
              ## It was a numeric argument, we should use it in the next
              ## plot.
              next_x = s;
              next_x_set = 1;
            else
              s_set = 1;
            endif
          endif
          
          if (! s_set),
            ## set default format
            fmt_mat = default_fmt;
          else
            
            ## s is a format argument (a line-specification or a
            ## switch). s can have more than one row (useful when
            ## plotting matrices). Even if one row of s is a
            ## line-specification, another row can be a switch. We have
            ## to decode every row of s. If there are more arguments,
            ## theese can also be format arguments, so we have to check
            ## them too. It would be nice to make this decoding in a
            ## separate function, but we don't know how many arguments
            ## that should be sent to that function (all_va_args is too
            ## much, so we would need to skip some arguments).
            
            ## The function 
            ## [status, style, indices, val] = \
            ##                           __mjpltopt__ (caller, fmt, arg)
            ## takes three input arguments:
            ##
            ## caller:  The name of the caller-function (used for error
            ##          messages).
            ## fmt:     A line-specification or a switch (or a
            ##          combination).
            ## arg:     Switch-argument (optional).
            ## 
            ## and returns four arguments:
            ## 
            ## status:  An integer
            ##          0: __mjpltopt__ used only two arguments.
            ##          1: fmt is a switch and __mjpltopt__ needs three
            ##             arguments (but only got two). 
            ##          2: fmt is a switch and __mjpltopt__ got three
            ##             arguments (__mjpltopt__ used all three
            ##             arguments).
            ## style:   A matrix of integers representing the plot styles
            ##          (lines, points, linespoints etc).
            ## indices: A matrix of integers representing the indices in
            ##          the variable val (below). 
            ##          column 1: linetype startindex in val
            ##                 2: linetype stopindex in val
            ##                 3: linewidth startindex
            ##                 4: linewidth stopindex
            ##                 5: pointtype startindex
            ##                 6: pointtype stopindex
            ##                 7: pointsize startindex
            ##                 8: pointsize stopindex
            ##                 9: key startindex
            ##                10: key stopindex
            ##                11: smooth startindex
            ##                12: smooth stopindex
            ##                13: using startindex
            ##                14: using stopindex
            ##                15: axes startindex
            ##                16: axes stopindex
            ## val:     String vector containing the values for the
            ##          linetype, linewidth etc.
            
            if (nargin >= va_ind),
              ## We shall send three arguments (caller, s and arg) to
              ## __mjpltopt__, arg can be a switch-argument or a numeric
              ## argument that shall be plotted, but we don't know yet.
              arg = va_arg;
              ++va_ind;
              send3args = 1;
              [status, style, indices, val] = __mjpltopt__ (caller, \
                                                            s, \
                                                            arg);
            else
              ## s was the last argument to mjplot, we can only send two
              ## arguments to __mjpltopt__.
              send3args = 0;
              arg_is_sa = 0;
              [status, style, indices, val] = __mjpltopt__ (caller, s);
              if (status == 1),
                error ("%s: missing argument", caller);
              endif
            endif
            
            ## Now, we have decoded the first format argument. We have to
            ## test if there are more format arguments. If we sent three
            ## arguments to __mjpltopt__ and status is 0, we have to test
            ## if arg is a line spesification or a switch. If we sent
            ## three arguments to __mjpltopt__ and status is 2, we have to
            ## check if there are more arguments.
            while (1)
              if ((send3args == 0) || (status == 2 && nargin < va_ind)),
                ## We have used all arguments to mjplot.
                break;
              elseif (status == 2 && nargin > va_ind),
                ## We have two or more arguments left, we have to test if
                ## the next argument is a line-specification or a
                ## switch.
                send3args = 1;
                s = va_arg;
                ++va_ind;
                if (! isstr (s)),
                  ## It was a numeric argument (not a line-spec or a 
                  ## switch).
                  next_x = s;
                  next_x_set = 1;
                  break;
                endif
                arg = va_arg;
                ++va_ind;
              elseif (status == 2 && nargin == va_ind),
                ## We have one argument left, check if it is a
                ## line-specification or a switch (a switch can have a
                ## default-value).
                send3args = 0;
                s = va_arg;
                ++va_ind;
                if (! isstr (s)),
                  ## It was a numeric argument (not a line-spec or switch).
                  next_x = s;
                  next_x_set = 1;
                  break;
                endif
              elseif (nargin >= va_ind),
                ## We haven't used arg yet (it wasn't a switch-argument). We
                ## have one or more arguments left that have to be tested.
                if (! isstr (arg)),
                  ## arg is a numeric argument (not a line-spec or switch).
                  next_x = arg;
                  next_x_set = 1;
                  break;
                endif
                ## arg is a line-specification or a switch.
                s = arg;
                send3args = 1;
                arg = va_arg;
                ++va_ind;
              elseif (nargin < va_ind),
                ## arg is the last argument. It can be a line-
                ## specification or a switch with a default value or it
                ## can be a numeric argument that shall be plotted.
                if (! isstr (arg)),
                  ## arg is a numeric argument (not a line-spec or switch).
                  next_x = arg;
                  next_x_set = 1;
                  break;
                endif
                s = arg;
                send3args = 0;
              else
                error ("Ops! Something went wrong here...");
              endif
              
              if (send3args),
                [status, style2, indices2, val2] = \
                    __mjpltopt__ (caller, s, arg);
              else
                [status, style2, indices2, val2] = \
                    __mjpltopt__ (caller, s);
                if (status == 1),
                  error ("%s: missing switch-argument", caller);
                endif
              endif
              
              r1 = rows (style);
              r2 = rows (style2);
              
              if (r1 > r2),
                style2 = [style2; \
                          repmat(style2(r2,:), \
                                 r1-r2, 1)];
                indices2 = [indices2; \
                            repmat(indices2(r2,:), \
                                   r1-r2, 1)];
              elseif (r2 > r1),
                style = [style; \
                         repmat(style(r1,:), \
                                r2-r1, 1)];
                indices = [indices; \
                           repmat(indices(r1,:), \
                                  r2-r1, 1)];
              endif
              tmp = find (indices2);
              indices(tmp) = indices2(tmp) + length (val);
              val = sprintf ("%s%s", val, val2);
              style = style | style2;
            endwhile
            
            default_style = deblank (STYLE(1,:));
            fmt_mat = "";
            ## Now create format string.
            if (isempty (val)),
              val = " ";
              ## Could be any string vector, it makes it legal to use
              ## single index. Eg val(1:0) gives the empty string.
            endif
            for n = 1:rows (style)
              
              ## Set style
              if ((style(n,1) && style(n,2)) || \
                  (style(n,1) && style(n,5))),
                ## lines and points -> linespoints or
                ## lines and dots   -> linespoints
                style(n,[1, 2, 3, 5]) = [0, 0, 1, 0];
              elseif (style(n,9) && style(n,13)),
                ## errorbars and boxes -> boxerrorbars
                style(n,[9, 13, 14]) = [0, 0, 1];
              endif
              style_no = find (style(n,:));
              if (prod (size (style_no)) > 1),
                error ("%s: to many styles set", caller);
              endif
              
              if (isempty (style_no)),
                fmt = sprintf ("%s %s", WITH, default_style);
              else
                fmt = sprintf ("%s %s", WITH, deblank (STYLE (style_no,:)));
              endif
              
              if (indices(n,1)),
                ## Set linetype
                fmt = sprintf ("%s %s %s", \
                               fmt, \
                               LINETYPE, \
                               val(indices(n,1):indices(n,2)));
              endif
              
              if (indices(n,3)),
                ## set linewidth
                fmt = sprintf ("%s %s %s", \
                               fmt, \
                               LINEWIDTH, \
                               val(indices(n,3):indices(n,4)));
              endif
              
              if (indices(n,5)),
                ## set pointtype
                fmt = sprintf ("%s %s %s", \
                               fmt, \
                               POINTTYPE, \
                               val(indices(n,5):indices(n,6)));
              endif
              
              if (indices(n,7)),
                ## set pointsize
                fmt = sprintf ("%s %s %s", \
                               fmt, \
                               POINTSIZE, \
                               val(indices(n,7):indices(n,8)));
              endif
              
              if (indices(n,9)),
                ## set key
                fmt = sprintf ("%s \"%s\" %s", \
                               TITLE, \
                               val(indices(n,9):indices(n,10)), \
                               fmt);
              endif
              
              if (indices(n,15)),
                ## set axes
                fmt = sprintf ("%s %s %s", \
                               AXES, \
                               val(indices(n,15):indices(n,16)), \
                               fmt);
              endif
              
              if (indices(n,11)),
                ## set smooth
                fmt = sprintf ("%s %s %s", \
                               SMOOTH, \
                               val(indices(n,11):indices(n,12)), \
                               fmt);
              endif
              
              if (indices(n,13)),
                ## set using
                fmt = sprintf ("%s %s %s", \
                               USING, \
                               val(indices(n,13):indices(n,14)), \
                               fmt);
              endif
              
              if (isempty (fmt_mat)),
                fmt_mat = fmt;
              else
                fmt_mat = [fmt_mat; fmt];
              endif
            endfor
          endif
          
          if (x_set && (! y_set)),
            __mjplt1__ (caller, x, fmt_mat);
          elseif (x_set && y_set),
            __mjplt2__ (caller, x, y, fmt_mat);
          endif
          x_set = 0;
          y_set = 0;
          s_set = 0;
        endwhile
      catch
        
        ## An error occurred. The global variables __mjplt_mjgraw_cmd__
        ## and __mjplt_il_data__ may have been changed; restore them.
        
        __mjplt_mjgraw_cmd__ = __mjplt_mjgraw_cmd__(1:lm);
        if (__mjplt_mjgraw_cmd__(length (__mjplt_mjgraw_cmd__)) == ","),
          __mjplt_mjgraw_cmd__(length (__mjplt_mjgraw_cmd__)) = "\n";
        endif
        if (li),
          __mjplt_il_data__ = __mjplt_il_data__(1:li);
        else
          __mjplt_il_data__ = "";
        endif
        error (__error_text__);
      end_try_catch
      
    endif
    
    if (__mjplt_mjgraw_cmd__(length (__mjplt_mjgraw_cmd__)) == ","),
      __mjplt_mjgraw_cmd__(length (__mjplt_mjgraw_cmd__)) = "\n";
    endif
    
    if ((! strcmp (__mjplt_mjgraw_cmd__, "plot")) && \
        (! isempty (__mjplt_il_data__))),
      ##sprintf ("%s%s", __mjplt_mjgraw_cmd__, __mjplt_il_data__)
      mjgraw ("\nset nologscale\n");
      mjgraw ("set nopolar\n");
      mjgraw (sprintf ("%s%s", __mjplt_mjgraw_cmd__, __mjplt_il_data__));
    else
      error ("mjplot: nothing to plot");
    endif
  unwind_protect_cleanup
    do_fortran_indexing = dfi;
    empty_list_elements_ok = eleo;
  end_unwind_protect
endfunction
