## function b = repmat (a, m, n)
##
## Replicate and tile a matrix.
##
## b = repmat (a, m), where a is a matrix and m is an integer:
## Replicates and tiles the matrix a to produce the m-by-m block matrix
## b.
##
## b = repmat (a, [m, n]) or
## b = repmat (a, m, n), where a is a matrix and  m and n are integers:
## Replicates and tiles the matrix a to produce the m-by-n block matrix
## b.


function b = repmat (a, m, n)
  if nargin < 2
    error("repmat: expecting at least two arguments")
  elseif nargin == 2
    if (prod (size (m)) == 1),
      blocksize = [m, m];
    else
      blocksize = m;
    endif
  else
    blocksize = [m, n];
  endif

  [ar, ac] = size (a);
  if  max ([ar, ac]) == 1
    if isstr (a)
      b = setstr (toascii (a) * ones (blocksize));
    else
      b = a * ones (blocksize);
    endif
  else
    arind = (1:ar)';
    acind = (1:ac)';
    arind = arind(:, ones (1, blocksize(1)));
    acind = acind(:, ones (1, blocksize(2)));
    b = a(arind, acind);
  endif
endfunction
