## This function is used to test some functionalities of mjplot (to see
## if you broke something when you rewrote the code).
##
## Usage: For all tests: testmjplot ();
##        For test number 2 and 5: testmjplot ([2, 5]);

## Mats Jansson, May 9 2000 - June 7 2000
## Report bugs to: mats@nera.no

function testmjplot (arg)
  global __mjplt_mjgraw_cmd__ = "plot";
  global __mjplt_il_data__ = "";

  hold off
  mjcloseplot ();
  if (! nargin),
    arg = 1:100;
  endif

  testno = 0;

  if (any (any (arg == ++testno))),
    ## Test if numeric arguments (scalars, vectors and matrices, real
    ## and complex) are parsed the right way.
    fputs (stdout, sprintf ("test %d: ", testno));
    fflush (stdout);
    mjplot (5, "@;single real scalar;",
            2+5*i, "@;single complex scalar;",
            3, 5, "@;real scalar vs real scalar;",
            4+i, 5, "@;real scalar vs complex scalar;",
            5, 5+i, "@;complex scalar vs real scalar;",
            6+i, 5+i, "@;complex scalar vs complex scalar;",
            [1, 2, 1], ";single real row vector;",
            [1; 3; 1], ";single real column vector;",
            [4, 5, 6] + i*[1, 2, 1], ";single complex row vector;",
            [4; 5; 6] + i*[1; 3; 1], ";single complex column vector;",
            [7, 8, 9], [1, 2, 1], ";row vector vs row vector;",
            [7, 8, 9], [1; 3; 1], ";column vector vs row vector;",
            [7; 8; 9], [1, 4, 1], ";row vector vs column vector;",
            [7; 8; 9], [1; 5; 1], ";column vector vs column vector;",
            [10, 11, 12], [1, 2, 1; 1, 3, 1],
            ";rows of matrix vs row vector;",
            [10, 11, 12], [1, 4, 1; 1, 5, 1]',
            ";columns of matrix vs row vector;",
            [10; 11; 12], [1, 6, 1; 1, 7, 1],
            ";rows of matrix vs column vector;",
            [10; 11; 12], [1, 8, 1; 1, 9, 1]',
            ";columns of matrix vs column vector;",
            [13, 14, 15], [1, 2, 1; 1, 3, 1; 1, 4, 1]',
            ";columns of matrix vs row vector (rows and columns match);",
            [16, 17, 18; 19, 20, 21], [1, 2, 1],
            ";row vector vs rows of matrix;",
            [16, 17, 18; 19, 20, 21], [1; 3; 1],
            ";column vector vs rows of matrix;",
            [16, 17, 18; 19, 20, 21]', [1, 4, 1],
            ";row vector vs columns of matrix;",
            [16, 17, 18; 19, 20, 21]', [1; 5; 1],
            ";column vector vs columns of matrix;",
            [22, 23, 24; 25, 26, 27; 28, 29, 30]', [1, 2, 1],
            ";row vector vs columns of matrix (rows and columns match);",
            [31, 32, 33; 34, 35, 36; 37, 38, 39]',
            [1, 2, 1; 1, 3, 1; 1, 4, 1]',
            ";columns of matrix vs columns of matrix;",
            [40, 41, 42; 43, 44, 45]', [1, 2, 1; 1, 3, 1]',
            ";real matrix vs real matrix;")
    if (! strcmp (__mjplt_mjgraw_cmd__,
                  "plot '-' t \"single real scalar\" w p, '-' t \"single complex scalar\" w p, '-' t \"real scalar vs real scalar\" w p, '-' t \"real scalar vs complex scalar\" w p, '-' t \"complex scalar vs real scalar\" w p, '-' t \"complex scalar vs complex scalar\" w p, '-' t \"single real row vector\" w l, '-' t \"single real column vector\" w l, '-' t \"single complex row vector\" w l, '-' t \"single complex column vector\" w l, '-' t \"row vector vs row vector\" w l, '-' t \"column vector vs row vector\" w l, '-' t \"row vector vs column vector\" w l, '-' t \"column vector vs column vector\" w l, '-' t \"rows of matrix vs row vector\" w l, '-' t \"rows of matrix vs row vector\" w l, '-' t \"columns of matrix vs row vector\" w l, '-' t \"columns of matrix vs row vector\" w l, '-' t \"rows of matrix vs column vector\" w l, '-' t \"rows of matrix vs column vector\" w l, '-' t \"columns of matrix vs column vector\" w l, '-' t \"columns of matrix vs column vector\" w l, '-' t \"columns of matrix vs row vector (rows and columns match)\" w l, '-' t \"columns of matrix vs row vector (rows and columns match)\" w l, '-' t \"columns of matrix vs row vector (rows and columns match)\" w l, '-' t \"row vector vs rows of matrix\" w l, '-' t \"row vector vs rows of matrix\" w l, '-' t \"column vector vs rows of matrix\" w l, '-' t \"column vector vs rows of matrix\" w l, '-' t \"row vector vs columns of matrix\" w l, '-' t \"row vector vs columns of matrix\" w l, '-' t \"column vector vs columns of matrix\" w l, '-' t \"column vector vs columns of matrix\" w l, '-' t \"row vector vs columns of matrix (rows and columns match)\" w l, '-' t \"row vector vs columns of matrix (rows and columns match)\" w l, '-' t \"row vector vs columns of matrix (rows and columns match)\" w l, '-' t \"columns of matrix vs columns of matrix\" w l, '-' t \"columns of matrix vs columns of matrix\" w l, '-' t \"columns of matrix vs columns of matrix\" w l, '-' t \"real matrix vs real matrix\" w l, '-' t \"real matrix vs real matrix\" w l\n") ||
        ! strcmp (__mjplt_il_data__,
                  "1 5\ne\n2 5\ne\n3 5\ne\n4 5\ne\n5 5\ne\n6 5\ne\n1 1\n2 2\n3 1\ne\n1 1\n2 3\n3 1\ne\n4 1\n5 2\n6 1\ne\n4 1\n5 3\n6 1\ne\n7 1\n8 2\n9 1\ne\n7 1\n8 3\n9 1\ne\n7 1\n8 4\n9 1\ne\n7 1\n8 5\n9 1\ne\n10 1\n11 2\n12 1\ne\n10 1\n11 3\n12 1\ne\n10 1\n11 4\n12 1\ne\n10 1\n11 5\n12 1\ne\n10 1\n11 6\n12 1\ne\n10 1\n11 7\n12 1\ne\n10 1\n11 8\n12 1\ne\n10 1\n11 9\n12 1\ne\n13 1\n14 2\n15 1\ne\n13 1\n14 3\n15 1\ne\n13 1\n14 4\n15 1\ne\n16 1\n17 2\n18 1\ne\n19 1\n20 2\n21 1\ne\n16 1\n17 3\n18 1\ne\n19 1\n20 3\n21 1\ne\n16 1\n17 4\n18 1\ne\n19 1\n20 4\n21 1\ne\n16 1\n17 5\n18 1\ne\n19 1\n20 5\n21 1\ne\n22 1\n23 2\n24 1\ne\n25 1\n26 2\n27 1\ne\n28 1\n29 2\n30 1\ne\n31 1\n32 2\n33 1\ne\n34 1\n35 3\n36 1\ne\n37 1\n38 4\n39 1\ne\n40 1\n41 2\n42 1\ne\n43 1\n44 3\n45 1\ne\n")),
      disp ("fail") 
    else
      disp ("ok")
    endif
    fflush (stdout);
    ##input ("Press return", 1);
  endif
  
  if (any (any (arg == ++testno))),
    ## Test that the last numeric argument is plotted when it appears
    ## after a line specification.
    fputs (stdout, sprintf ("test %d: ", testno));
    fflush (stdout);
    mjplot (2:4, "-@", 3:5)
    if (! strcmp (__mjplt_mjgraw_cmd__, "plot '-' w linesp, '-' w l\n") ||
        ! strcmp (__mjplt_il_data__, "1 2\n2 3\n3 4\ne\n1 3\n2 4\n3 5\ne\n")),
      disp ("fail") 
    else
      disp ("ok")
    endif
    fflush (stdout);
    ##input ("Press return", 1);
  endif

  if (any (any (arg == ++testno))),
    ## Test that the last numeric argument is taken as a switch argument
    ## if it appears after a switch.
    fputs (stdout, sprintf ("test %d: ", testno));
    fflush (stdout);
    mjplot (2:4, "LW", 3)
    if (! strcmp (__mjplt_mjgraw_cmd__, "plot '-' w l lw 3\n") ||
        ! strcmp (__mjplt_il_data__, "1 2\n2 3\n3 4\ne\n")),
      disp ("fail")
    else
      disp ("ok")
    endif
    fflush (stdout);
    ##input ("Press return", 1);
  endif
  
  if (any (any (arg == ++testno))),
    ## Test that the last argument can be a switch with a default value.
    fputs (stdout, sprintf ("test %d: ", testno));
    fflush (stdout);
    mjplot ([1, 2, 1], "Smooth")
    if (! strcmp (__mjplt_mjgraw_cmd__, "plot '-' s c w l\n") ||
        ! strcmp (__mjplt_il_data__, "1 1\n2 2\n3 1\ne\n")),
      disp ("fail") 
    else
      disp ("ok")
    endif
    fflush (stdout);
    ##input ("Press return", 1);
  endif
  
  if (any (any (arg == ++testno))),
    ## Test that the empty string or the empty matrix as a
    ## switch-argument gives the default value. Also test that the
    ## empty string as a line specification gives the default format.
    fputs (stdout, sprintf ("test %d: ", testno));
    fflush (stdout);
    mjplot (2:4, "Style", "", 3:5, "Style", [], 4:6, "")
    if (! strcmp (__mjplt_mjgraw_cmd__, "plot '-' w l, '-' w l, '-' w l\n") ||
        ! strcmp (__mjplt_il_data__, "1 2\n2 3\n3 4\ne\n1 3\n2 4\n3 5\ne\n1 4\n2 5\n3 6\ne\n")),
      disp ("fail") 
    else
      disp ("ok")
    endif
    fflush (stdout);
    ##input ("Press return", 1);
  endif
  
  if (any (any (arg == ++testno))),
    ## Test some line specifications
    fputs (stdout, sprintf ("test %d: ", testno));
    fflush (stdout);
    mjplot ([1, 2, 1], "-",
            [1, 3, 1], ".",
            [1, 4, 1], ":",
            [1, 5, 1], "@",
            [1, 6, 1], "-@",
            [1, 7, 1], "@-",
            [1, 8, 1], "-.",
            [1, 9, 1], ".-",
            [1, 10, 1], "L",
            [1, 11, 1], "#",
            [1, 12, 1], "i",
            [4, 5, 6], [1, 2, 1], "or",
            [4, 5, 6], [1, 3, 1], "dg",
            [4, 5, 6], [1, 4, 1], "+b",
            [4, 5, 6], [1, 5, 1], "sm",
            [4, 5, 6], [1, 6, 1], "xc",
            [4, 5, 6], [1, 7, 1], "^k",
            [4, 5, 6], [1, 8, 1], "*w",
            [4, 5, 6], [1, 9, 1], "<y",
            [4, 5, 6], [1, 10, 1], ">-",
            [4, 5, 6], [1, 11, 1], "h1",
            [4, 5, 6], [1, 12, 1], "p2",
            [4, 5, 6], [1, 13, 1], "v3;;;"
            )
    if (! strcmp (__mjplt_mjgraw_cmd__, "plot '-' w l, '-' w d, '-' w d, '-' w p, '-' w linesp, '-' w linesp, '-' w linesp, '-' w linesp, '-' w st, '-' w boxes, '-' w i, '-' w p lt 1 pt 1, '-' w p lt 2 pt 1, '-' w p lt 3 pt 2, '-' w p lt 4 pt 3, '-' w p lt 5 pt 4, '-' w p lt 6 pt 5, '-' w p lt 6 pt 6, '-' w p lt 6 pt 6, '-' w linesp pt 6, '-' w p lt 1 pt 6, '-' w p lt 2 pt 6, '-' t \";\" w p lt 3 pt 6\n") ||
        ! strcmp (__mjplt_il_data__, "1 1\n2 2\n3 1\ne\n1 1\n2 3\n3 1\ne\n1 1\n2 4\n3 1\ne\n1 1\n2 5\n3 1\ne\n1 1\n2 6\n3 1\ne\n1 1\n2 7\n3 1\ne\n1 1\n2 8\n3 1\ne\n1 1\n2 9\n3 1\ne\n1 1\n2 10\n3 1\ne\n1 1\n2 11\n3 1\ne\n1 1\n2 12\n3 1\ne\n4 1\n5 2\n6 1\ne\n4 1\n5 3\n6 1\ne\n4 1\n5 4\n6 1\ne\n4 1\n5 5\n6 1\ne\n4 1\n5 6\n6 1\ne\n4 1\n5 7\n6 1\ne\n4 1\n5 8\n6 1\ne\n4 1\n5 9\n6 1\ne\n4 1\n5 10\n6 1\ne\n4 1\n5 11\n6 1\ne\n4 1\n5 12\n6 1\ne\n4 1\n5 13\n6 1\ne\n")),
      disp ("fail") 
    else
      disp ("ok")
    endif
    fflush (stdout);
    ##input ("Press return", 1);
  endif

  if (any (any (arg == ++testno))),
    ## Test the Style switch
    fputs (stdout, sprintf ("test %d: ", testno));
    fflush (stdout);
    mjplot ([ones(1,28); (2:29); ones(1,28)], 
            ["Style    ";
             "St       ";
             "LineStyle";
             "LineS    ";
             "LS       "],
            ["-          ";
             "--         ";
             "l          ";
             "lines      ";
             "           ";
             "@          ";
             "p          ";
             "points     ";
             "-@         ";
             "@-         ";
             "-.         ";
             ".-         ";
             "lp         ";
             "linesp     ";
             "linespoints";
             "i          ";
             "^          ";
             "imp        ";
             "impulses   ";
             ".          ";
             ":          ";
             "do         ";
             "dots       ";
             "L          ";
             "ste        ";
             "steps      ";
             "#          ";
             "boxes      "],
            [4, 5, 6], [1, 2, 1], "Style", "",
            [4, 5, 6], [1, 3, 1], "Style", [],
            [4, 5, 6], [1, 4, 1], "Style")
    if (! strcmp (__mjplt_mjgraw_cmd__, "plot '-' w l, '-' w l, '-' w l, '-' w l, '-' w l, '-' w p, '-' w p, '-' w p, '-' w linesp, '-' w linesp, '-' w linesp, '-' w linesp, '-' w linesp, '-' w linesp, '-' w linesp, '-' w i, '-' w i, '-' w i, '-' w i, '-' w d, '-' w d, '-' w d, '-' w d, '-' w st, '-' w st, '-' w st, '-' w boxes, '-' w boxes, '-' w l, '-' w l, '-' w l\n") ||
        ! strcmp (__mjplt_il_data__, "1 1\n2 2\n3 1\ne\n1 1\n2 3\n3 1\ne\n1 1\n2 4\n3 1\ne\n1 1\n2 5\n3 1\ne\n1 1\n2 6\n3 1\ne\n1 1\n2 7\n3 1\ne\n1 1\n2 8\n3 1\ne\n1 1\n2 9\n3 1\ne\n1 1\n2 10\n3 1\ne\n1 1\n2 11\n3 1\ne\n1 1\n2 12\n3 1\ne\n1 1\n2 13\n3 1\ne\n1 1\n2 14\n3 1\ne\n1 1\n2 15\n3 1\ne\n1 1\n2 16\n3 1\ne\n1 1\n2 17\n3 1\ne\n1 1\n2 18\n3 1\ne\n1 1\n2 19\n3 1\ne\n1 1\n2 20\n3 1\ne\n1 1\n2 21\n3 1\ne\n1 1\n2 22\n3 1\ne\n1 1\n2 23\n3 1\ne\n1 1\n2 24\n3 1\ne\n1 1\n2 25\n3 1\ne\n1 1\n2 26\n3 1\ne\n1 1\n2 27\n3 1\ne\n1 1\n2 28\n3 1\ne\n1 1\n2 29\n3 1\ne\n4 1\n5 2\n6 1\ne\n4 1\n5 3\n6 1\ne\n4 1\n5 4\n6 1\ne\n")),
      disp ("fail") 
    else
      disp ("ok")
    endif
    fflush (stdout);
    ##input ("Press return", 1);
  endif

  if (any (any (arg == ++testno))),
    ## Test the LineWidth switch.
    fputs (stdout, sprintf ("test %d: ", testno));
    fflush (stdout);
    mjplot ([ones(1,3); (2:4); ones(1,3)], 
            ["LineWidth";  ## Full switch name
             "LineW    ";  ## Abbreviation
             "LW       "], ## Another full switch-name
            ["2        "; 
             "         ";  ## Ignore LineWidth
             "4, 5, 6  "], ## Use the first element (4)
            [2, 3, 4], [1, 2, 1], "LW", "", ## Ignore LineWidth
            [3, 4, 5], [1, 2, 1], "LW", [], ## Ignore LineWidth
            [4, 5, 6], [1, 2, 1], "LW", 6,  ## Scalar numeric argument
            [5, 6, 7], [1, 2, 1], "LW", 
            [6, 1, 2]) ## Use the first element (6)
    if (! strcmp (__mjplt_mjgraw_cmd__, "plot '-' w l lw 2, '-' w l, '-' w l lw 4, '-' w l, '-' w l, '-' w l lw 6, '-' w l lw 6\n") ||
        ! strcmp (__mjplt_il_data__, "1 1\n2 2\n3 1\ne\n1 1\n2 3\n3 1\ne\n1 1\n2 4\n3 1\ne\n2 1\n3 2\n4 1\ne\n3 1\n4 2\n5 1\ne\n4 1\n5 2\n6 1\ne\n5 1\n6 2\n7 1\ne\n")),
      disp ("fail") 
    else
      disp ("ok")
    endif
    fflush (stdout);
    ##input ("Press return", 1);
  endif

  if (any (any (arg == ++testno))),
    ## Test the PointType switch. Also test that if there are more lines
    ## in the switch-argument than in the switch, the last line in the
    ## switch are used for the rest of the  switch-arguments.
    fputs (stdout, sprintf ("test %d: ", testno));
    fflush (stdout);
    mjplot ([ones(1,23); (2:24); ones(1,23)], 
            ["PointType";  ## Full switch-name
             "PT       ";  ## Another full switch-name
             "Marker   ";  ## A third full switch-name
             "PointT   "], ## Abbreviation
            ["1        ";
             "         ";  ## Ignore PointType
             "3        ";
             "4, 5, 6  ";  ## Use the first element (4)
             "o        ";
             "d        ";  ## Abbreviation
             "diamond  ";
             "+        ";
             "s        ";  ## Abbreviation
             "square   ";
             "x        ";
             "^        ";
             "*        ";
             ".        ";
             "<        ";
             ">        ";
             "h        ";  ## Abbreviation
             "hexagram ";  
             "p        ";  ## Abbreviation
             "pentagram";
             "v        ";
             "n        ";  ## Abbreviation
             "none     "],
            [2, 3, 4], [1, 2, 1], "PT", "", ## Ignore PointType
            [3, 4, 5], [1, 2, 1], "PT", [], ## Ignore PointType
            [4, 5, 6], [1, 2, 1], "PT", 1,
            [4, 5, 6], [2, 3, 2], "PT", [2, 3, 4])
            
    if (! strcmp (__mjplt_mjgraw_cmd__, "plot '-' w p pt 1, '-' w l, '-' w p pt 3, '-' w p pt 4, '-' w p pt 1, '-' w p pt 1, '-' w p pt 1, '-' w p pt 2, '-' w p pt 3, '-' w p pt 3, '-' w p pt 4, '-' w p pt 5, '-' w p pt 6, '-' w p pt 6, '-' w p pt 6, '-' w p pt 6, '-' w p pt 6, '-' w p pt 6, '-' w p pt 6, '-' w p pt 6, '-' w p pt 6, '-' w l, '-' w l, '-' w l, '-' w l, '-' w p pt 1, '-' w p pt 2\n") ||
        ! strcmp (__mjplt_il_data__, "1 1\n2 2\n3 1\ne\n1 1\n2 3\n3 1\ne\n1 1\n2 4\n3 1\ne\n1 1\n2 5\n3 1\ne\n1 1\n2 6\n3 1\ne\n1 1\n2 7\n3 1\ne\n1 1\n2 8\n3 1\ne\n1 1\n2 9\n3 1\ne\n1 1\n2 10\n3 1\ne\n1 1\n2 11\n3 1\ne\n1 1\n2 12\n3 1\ne\n1 1\n2 13\n3 1\ne\n1 1\n2 14\n3 1\ne\n1 1\n2 15\n3 1\ne\n1 1\n2 16\n3 1\ne\n1 1\n2 17\n3 1\ne\n1 1\n2 18\n3 1\ne\n1 1\n2 19\n3 1\ne\n1 1\n2 20\n3 1\ne\n1 1\n2 21\n3 1\ne\n1 1\n2 22\n3 1\ne\n1 1\n2 23\n3 1\ne\n1 1\n2 24\n3 1\ne\n2 1\n3 2\n4 1\ne\n3 1\n4 2\n5 1\ne\n4 1\n5 2\n6 1\ne\n4 2\n5 3\n6 2\ne\n")),
      disp ("fail") 
    else
      disp ("ok")
    endif
    fflush (stdout);
    ##input ("Press return", 1);
  endif

  if (any (any (arg == ++testno))),
    ## Test the PointSize switch. And test that if there are more rows
    ## in the switch than in the switch-argument, the last row of the
    ## switch-argument are used.
    fputs (stdout, sprintf ("test %d: ", testno));
    fflush (stdout);
    mjplot ([1:5; 2:6; 1:5], 
            ["PointSize ";  ## Full switch-name
             "MarkerSize";  ## Another full switch-name
             "PS        ";  ## A third full switch-name
             "PointS    ";  ## Abbreviation
             "MarkerS   "], ## Abbreviation
            ["1        "; 
             "         ";  ## Ignore LineWidth
             "4, 5, 6  "], ## Use the first element (4)
            [4, 5, 6], [1, 2, 1], "PS", "", ## Ignore PointSize
            [4, 5, 6], [2, 3, 2], "PS", [], ## Ignore PointSize
            [4, 5, 6], [3, 4, 3], "PS", 6,  ## Scalar numeric argument
            [4, 5, 6], [4, 5, 4], "PS", 
            [20, 1, 2]) ## Use the first element (6)
    if (! strcmp (__mjplt_mjgraw_cmd__, "plot '-' w p ps 1, '-' w l, '-' w p ps 4, '-' w p ps 4, '-' w p ps 4, '-' w l, '-' w l, '-' w p ps 6, '-' w p ps 20\n") ||
        ! strcmp (__mjplt_il_data__, "1 1\n2 2\n3 1\ne\n1 2\n2 3\n3 2\ne\n1 3\n2 4\n3 3\ne\n1 4\n2 5\n3 4\ne\n1 5\n2 6\n3 5\ne\n4 1\n5 2\n6 1\ne\n4 2\n5 3\n6 2\ne\n4 3\n5 4\n6 3\ne\n4 4\n5 5\n6 4\ne\n")),
      disp ("fail") 
    else
      disp ("ok")
    endif
    fflush (stdout);
    ##input ("Press return", 1);
  endif

  if (any (any (arg == ++testno))),
    ## Test the Title switch.
    fputs (stdout, sprintf ("test %d: ", testno));
    fflush (stdout);
    mjplot ([1:3; 2:4; 1:3], 
            ["Title";   ## Full switch-name
             "T    "],  ## Abbreviation
            ["First title "; 
             "Second title";
             "            "], ## No title
            [4, 5, 6], [1, 2, 1], "T", "",  ## No title
            [4, 5, 6], [2, 3, 2], "T", [])  ## No title
    if (! strcmp (__mjplt_mjgraw_cmd__, "plot '-' t \"First title\" w l, '-' t \"Second title\" w l, '-' t \"\" w l, '-' t \"\" w l, '-' t \"\" w l\n") ||
        ! strcmp (__mjplt_il_data__, "1 1\n2 2\n3 1\ne\n1 2\n2 3\n3 2\ne\n1 3\n2 4\n3 3\ne\n4 1\n5 2\n6 1\ne\n4 2\n5 3\n6 2\ne\n")),
      disp ("fail") 
    else
      disp ("ok")
    endif
    fflush (stdout);
    ##input ("Press return", 1);
  endif

  if (any (any (arg == ++testno))),
    ## Test the LineType switch.
    fputs (stdout, sprintf ("test %d: ", testno));
    fflush (stdout);
    mjplot ([1:27; 2:28; 1:27], 
            ["LineType       ";  ## Full switch-name
             "LineT          ";  ## Abbreviation
             "LT             ";  ## Full switch-name
             "Color          ";  ## Full switch-name
             "Col            ";  ## Abbreviation
             "MarkerEdgeColor";  ## Full switch-name
             "MarkerE        ";  ## Abbreviation
             "MarkerFaceColor";  ## Full switch-name
             "MarkerF        "], ## Abbreviation
            ["0"; 
             "1";
             "2";
             "0, 0, 0";
             "1, 0, 0";
             "0, 1, 0";
             "red    ";
             "r      ";
             "green  ";
             "g      ";
             "b      ";
             "blu    ";
             "blue   ";
             "m      ";
             "magenta";
             "white  ";
             "w      ";
             "k      ";
             "black  ";
             "bla    ";
             "yellow ";
             "y      ";
             "brown  ";
             "br     ";
             "       ";
             "auto   ";
             "none   "],
            [4, 5, 6], [1, 2, 1], "LT", "",
            [4, 5, 6], [2, 3, 2], "LT", [],
            [4, 5, 6], [3, 4, 3], "LT", 0,
            [4, 5, 6], [4, 5, 4], "LT", [0.4, 0.4, 1])
    if (! strcmp (__mjplt_mjgraw_cmd__, "plot '-' w l lt 0, '-' w l lt 1, '-' w l lt 2, '-' w l lt 0, '-' w l lt 1, '-' w l lt 2, '-' w l lt 1, '-' w l lt 1, '-' w l lt 2, '-' w l lt 2, '-' w l lt 3, '-' w l lt 3, '-' w l lt 3, '-' w l lt 4, '-' w l lt 4, '-' w l lt 6, '-' w l lt 6, '-' w l lt 6, '-' w l lt 6, '-' w l lt 6, '-' w l lt 6, '-' w l lt 6, '-' w l lt 6, '-' w l lt 6, '-' w l, '-' w l, '-' w l, '-' w l, '-' w l, '-' w l lt 0, '-' w l lt 3\n") ||
        ! strcmp (__mjplt_il_data__, "1 1\n2 2\n3 1\ne\n1 2\n2 3\n3 2\ne\n1 3\n2 4\n3 3\ne\n1 4\n2 5\n3 4\ne\n1 5\n2 6\n3 5\ne\n1 6\n2 7\n3 6\ne\n1 7\n2 8\n3 7\ne\n1 8\n2 9\n3 8\ne\n1 9\n2 10\n3 9\ne\n1 10\n2 11\n3 10\ne\n1 11\n2 12\n3 11\ne\n1 12\n2 13\n3 12\ne\n1 13\n2 14\n3 13\ne\n1 14\n2 15\n3 14\ne\n1 15\n2 16\n3 15\ne\n1 16\n2 17\n3 16\ne\n1 17\n2 18\n3 17\ne\n1 18\n2 19\n3 18\ne\n1 19\n2 20\n3 19\ne\n1 20\n2 21\n3 20\ne\n1 21\n2 22\n3 21\ne\n1 22\n2 23\n3 22\ne\n1 23\n2 24\n3 23\ne\n1 24\n2 25\n3 24\ne\n1 25\n2 26\n3 25\ne\n1 26\n2 27\n3 26\ne\n1 27\n2 28\n3 27\ne\n4 1\n5 2\n6 1\ne\n4 2\n5 3\n6 2\ne\n4 3\n5 4\n6 3\ne\n4 4\n5 5\n6 4\ne\n")),
      disp ("fail") 
    else
      disp ("ok")
    endif
    fflush (stdout);
    ##input ("Press return", 1);
  endif

  if (any (any (arg == ++testno))),
    ## Test the Smooth switch.
    fputs (stdout, sprintf ("test %d: ", testno));
    fflush (stdout);
    mjplot ([1:5; 2:6; 3:7; 1:5], 
            ["Smooth";  ## Full switch-name
             "Smo   "], ## Abbreviation
            ["unique"; 
             "csplines";
             "        ";
             "bezier";
             "sbezier"],
            
            [4, 5, 6], [1, 2, 1], "Sm", "",
            [4, 5, 6], [2, 3, 2], "Sm", [])
    if (! strcmp (__mjplt_mjgraw_cmd__, "plot '-' s u w l, '-' s c w l, '-' s c w l, '-' s b w l, '-' s s w l, '-' s c w l, '-' s c w l\n") ||
        ! strcmp (__mjplt_il_data__, "1 1\n2 2\n3 3\n4 1\ne\n1 2\n2 3\n3 4\n4 2\ne\n1 3\n2 4\n3 5\n4 3\ne\n1 4\n2 5\n3 6\n4 4\ne\n1 5\n2 6\n3 7\n4 5\ne\n4 1\n5 2\n6 1\ne\n4 2\n5 3\n6 2\ne\n")),
      disp ("fail") 
    else
      disp ("ok")
    endif
    fflush (stdout);
    ##input ("Press return", 1);
  endif

  if (any (any (arg == ++testno))),
    ## Test the Axes switch.
    fputs (stdout, sprintf ("test %d: ", testno));
    fflush (stdout);
    mjgraw ("set xrange [0:20]\n");
    mjgraw ("set xtics border nomirror 0, 1, 10\n");
    mjgraw ("set yrange [0:100]\n");
    mjgraw ("set ytics border nomirror 0, 10, 50\n");
    mjgraw ("set x2range [0:40]\n");
    mjgraw ("set x2tics border nomirror 20, 5, 40\n");
    mjgraw ("set y2range [0:200]\n");
    mjgraw ("set y2tics border nomirror 100, 20, 200\n");
    
    mjplot ([0, 10], [0, 50], "Axes", "x1y1", "T", "x1y1",
            [0, 10], [200, 100], "Axes", "x1y2", "T", "x1y2",
            [20, 40], [50, 0], "Ax", "x2y1", "T", "x2y1",
            [20, 40], [100, 200], "Ax", "x2y2", "T", "x2y2",
            [0, 20], [50, 50], "Axes", "", 
            [10, 10], [0, 100], "Ax", [])
    if (! strcmp (__mjplt_mjgraw_cmd__, "plot '-' ax x1y1 t \"x1y1\" w l, '-' ax x1y2 t \"x1y2\" w l, '-' ax x2y1 t \"x2y1\" w l, '-' ax x2y2 t \"x2y2\" w l, '-' w l, '-' w l\n") ||
        ! strcmp (__mjplt_il_data__, "0 0\n10 50\ne\n0 200\n10 100\ne\n20 50\n40 0\ne\n20 100\n40 200\ne\n0 50\n20 50\ne\n10 0\n10 100\ne\n")),
      disp ("fail") 
    else
      disp ("ok")
    endif
    mjgraw ("set autoscale\n");
    mjgraw ("set xtics border mirror norotate autofreq\n");
    mjgraw ("set ytics border mirror norotate autofreq\n");
    mjgraw ("set nox2tics\n");
    mjgraw ("set noy2tics\n");
    fflush (stdout);
    ##input ("Press return", 1);
  endif

  if (any (any (arg == ++testno))),
    ## Test the Using switch.
    fputs (stdout, sprintf ("test %d: ", testno));
    fflush (stdout);
    mjplot ([0, 1, 4, 9, 16], "Using", [2, 1],
            [0, 0, 0, 0, 0], "Us", "(($1-1)*($1-1)):1", "@",
            [0, 1, 4, 9, 16], 1.1*[1, 2, 3, 4, 5], "Us", "",
            [0, 1, 4, 9, 16], 1.2*[1, 2, 3, 4, 5], "Us", [])
    if (! strcmp (__mjplt_mjgraw_cmd__, "plot '-' u 2:1 w l, '-' u (($1-1)*($1-1)):1 w p, '-' w l, '-' w l\n") ||
        ! strcmp (__mjplt_il_data__, "1 0\n2 1\n3 4\n4 9\n5 16\ne\n1 0\n2 0\n3 0\n4 0\n5 0\ne\n0 1.1\n1 2.2\n4 3.3\n9 4.4\n16 5.5\ne\n0 1.2\n1 2.4\n4 3.6\n9 4.8\n16 6\ne\n")),
      disp ("fail") 
    else
      disp ("ok")
    endif
    fflush (stdout);
    ##input ("Press return", 1);
  endif

  ##undo_string_escapes (__mjplt_mjgraw_cmd__)
  ##undo_string_escapes (__mjplt_il_data__)
  
endfunction
